; BMP2PI3 v1.1
;
; Converts 8-bit BMP images into Degas Elite PI3
; BMP files from Grafx2 works fine, not tested with others
;
; Width MUST be 640 pixels
; Height MUST be 400 lines
;
; ae@dhs.nu
; June 9, 2019
;
; Assemble with Devpac gen.ttp (or vasm with minor changes)

		comment	HEAD=%111
		output	.ttp

start:
		move.l	4(sp),a5			;address to basepage
		move.l	a5,d0				;basepage address
		add.l	#129,d0				;offset to commandline
		move.l	d0,filename			
		move.l	$0c(a5),d0			;length of text segment
		add.l	$14(a5),d0			;length of data segment
		add.l	$1c(a5),d0			;length of bss segment
		add.l	#$1000,d0			;length of stackpointer
		add.l	#$100,d0			;length of basepage
		move.l	a5,d1				;address to basepage
		add.l	d0,d1				;end of program
		and.l	#-2,d1				;make address even
		move.l	d1,sp				;new stackspace

		move.l	d0,-(sp)			;mshrink()
		move.l	a5,-(sp)
		move.w	d0,-(sp)
		move.w	#$4a,-(sp)
		trap	#1
		lea	12(sp),sp

		lea	text_head,a0
		bsr	cconws

		move.w	#$2f,-(sp)			;fgetdta()
		trap	#1
		addq.l	#2,sp
		move.l	d0,save_dta

		pea	dta				;fsetdta()
		move.w	#$1a,-(sp)
		trap	#1
		addq.l	#6,sp

		bsr	load
		bsr	convert_data
		bsr	save

exit:
		move.l	save_dta,-(sp)			;fsetdta()
		move.w	#$1a,-(sp)
		trap	#1
		addq.l	#6,sp

		lea	text_crlf,a0
		bsr	cconws
		lea	text_exit,a0
		bsr	cconws
		bsr	crawcin

		clr.w	-(sp)				;pterm()
		trap	#1

convert_data:
		lea	text_convert,a0
		bsr	cconws

		;read out bmp header (width/height)
		move.l	bmpaddr,a0
		move.l	18(a0),d0			;bmp width
		move.l	22(a0),d1			;bmp height

		swap	d0				;-> big endian
		swap	d1
		rol.w	#8,d0
		rol.w	#8,d1

		cmp.w	#640,d0
		beq.s	.xok
		lea	text_errdim,a0
		bsr	cconws
		bra	exit
.xok:
		cmp.w	#400,d1
		beq.s	.yok
		lea	text_errdim,a0
		bsr	cconws
		bra	exit
.yok:
		move.w	d0,width
		move.w	d1,height

		;reserve memory for st image
		move.l	#32034,-(sp)			;malloc()
		move.w	#72,-(sp)
		trap	#1
		addq.l	#6,sp
		tst.l	d0
		beq	exit				;not enough memory
		move.l	d0,staddr
		move.l	d0,a0
		move.l	#$00020000,(a0)+		;PI3 head
		move.w	#$0777,(a0)+

		;source and dest
		move.l	bmpaddr,a0
		lea	54+1024(a0),a0
		move.l	staddr,a1
		lea	34(a1),a1

		;calculate bottom line address, as BMP is turned upside down
		moveq	#0,d0
		moveq	#0,d1
		move.w	width,d0			;lw
		move.w	height,d1			;lines
		subq.w	#1,d1
		mulu	d0,d1				;ofs to last line
		add.l	d1,a0

		;do actual image conversion		
		move.w	height,d7
		subq.w	#1,d7
.y:		move.w	width,d6
		lsr.w	#4,d6
		subq.w	#1,d6
.x:		moveq	#16-1,d5
		moveq	#0,d1
.x2:
		lsl.w	#1,d1
		moveq	#0,d0
		move.b	(a0)+,d0
		beq.s	.nopix
		or.b	#%00000001,d1
.nopix:		dbra	d5,.x2
		;not.w	d1
		move.w	d1,(a1)+
		dbra	d6,.x
		sub.w	width,a0
		sub.w	width,a0
		dbra	d7,.y

		rts


load:
		lea	text_load,a0
		bsr	cconws
		move.l	filename,a0
		bsr	cconws
		lea	text_crlf,a0
		bsr	cconws

		clr.w	-(sp)				;fsfirst() get fileinfo
		move.l	filename,-(sp)
		move.w	#$4e,-(sp)
		trap	#1
		addq.l	#8,sp

		tst.l	d0
		beq.s	.ok
		lea	text_fnf,a0
		bsr	cconws
		bra	exit				;file not found

.ok:		move.l	dta+26,bmpsize			;filelength

		move.l	bmpsize,-(sp)			;malloc()
		move.w	#72,-(sp)
		trap	#1
		addq.l	#6,sp
		tst.l	d0
		beq	exit				;not enough memory
		move.l	d0,bmpaddr

		clr.w	-(sp)				;fopen()
		move.l	filename,-(sp)			
		move.w	#$3d,-(sp)
		trap	#1
		addq.l	#8,sp
		move.w	d0,fid

		move.l	bmpaddr,-(sp)			;fread()
		move.l	bmpsize,-(sp)
		move.w	fid,-(sp)
		move.w	#$3f,-(sp)
		trap	#1
		lea	12(sp),sp

		move.w	fid,-(sp)			;fclose()
		move.w	#$3e,-(sp)
		trap	#1
		addq.l	#4,sp
		rts

save:	
		;build ST-filename the same as bmp with different extension
		move.l	filename,a0
		lea	stfilename,a1
		moveq	#0,d0
		move.b	#8-1,d7
.loop:		move.b	(a0)+,d0
		cmp.b	#".",d0
		beq.s	.endfn
		move.b	d0,(a1)+
		dbra	d7,.loop
		bra	exit
.endfn:		move.b	d0,(a1)+
		move.b	#"p",(a1)+
		move.b	#"1",(a1)+
		move.b	#"3",(a1)+
		clr.b	(a1)+

		lea	text_save,a0
		bsr	cconws
		lea	stfilename,a0
		bsr	cconws
		lea	text_crlf,a0
		bsr	cconws

		clr.w	-(sp)				;fcreate()
		pea	stfilename
		move.w	#$3c,-(sp)
		trap	#1
		addq.w	#8,sp
		move.w	d0,fid

		move.l	staddr,-(sp)			;fwrite()
		move.l	#32034,-(sp)
		move.w	fid,-(sp)
		move.w	#$40,-(sp)
		trap	#1
		lea	12(sp),sp

		move.w	fid,-(sp)			;fclose()
		move.w	#$3e,-(sp)
		trap	#1
		addq.w	#4,sp

		rts

cconws:
;in: a0		address to null terminated text string
		move.l	a0,-(sp)
		move.w	#9,-(sp)
		trap	#1
		addq.l	#6,sp
		rts

crawcin:	move.w	#7,-(sp)
		trap	#1
		addq.l	#2,sp
		rts

		section	data

			;0123456789012345678901234567890123456789
text_head:	dc.b	"BMP2PI3 v1.00",13,10,13,10,0
text_load:	dc.b	"Loading file: ",0
text_convert:	dc.b	"Converting image...",13,10,0
text_save:	dc.b	"Saving file: ",0
text_fnf:	dc.b	"File not found! Exiting.",13,10,0
text_exit:	dc.b	"Program done, press any key to exit.",13,10,0
text_errdim:	dc.b	"Not a 640*400 8-bit BMP!",13,10,0
text_crlf:	dc.b	13,10,0
		even
		
		section	bss

stfilename:	ds.b	14
width:		ds.w	1
height:		ds.w	1
staddr:		ds.l	1
bmpaddr:	ds.l	1
bmpsize:	ds.l	1
stsize:		ds.l	1
save_dta:	ds.l	1
dta:		ds.l	11
filename:	ds.l	1
fid:		ds.w	1

